#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "prior.h"
#include "random.h"
#include "nrutil.h"
#include "sample.h"
#include "cutil.h"
#include "myio.h"
#include "simtree.h"
void getvalues(double prs[4], simmodel *m);
/***********************************************************************/
int main(int argc, char *argv[])
{
  simmodel m;
  poptree t;
  FILE *out;
  int i,j;
  char outfile[30],infile[30];
  double pars[7];
  
  if (argc==1) {
    printf("no arguments entered:\n using %s <infile> <out> <seed>\n",argv[0]);
    strcpy(infile,"infile");
    strcpy(outfile,"outfile");
  } else {
    printf("input line is ");
    for (i=0;i<argc;i++) printf("%s ",argv[i]);
    printf("\n");
    if ((argc<3)||(argc>4)) {
      fprintf(stderr,"use:\nsample infile outfile (seed)\n",argv[0]);  
      exit(0);
    }
    strcpy(infile,argv[1]);
    strcpy(outfile,argv[2]);
  }
	
  m = readsimmodel(infile);
  if (argc==4) m.seed = atoi(argv[3]);
  starTup(m.seed);
  out = openoutputfile(outfile);
  //  prepare the output file with some column headers
  fprintf(out,"mu N T L ");
  if (m.sizemodel) fprintf(out,"alpha ");
  if (m.sizemodel==2) fprintf(out,"beta gamma ");
  if (m.migmodel) {
    for (j=1;j<=m.npops;j++) fprintf(out,"p%d ",j);
    for (j=1;j<m.npops-1;j++) 
      fprintf(out,"s%d t%d ",j,j);
    fprintf(out,"t%d ",m.npops-1);	
  } 
  fprintf(out,"\n");
  // now get all the samples 
  for (i=1;i<=m.replicates;i++) {
    getvalues(pars,&m);			// get parameter values for this replicate
    m.theta=pars[2];
    t = simulate_poptree(m.ss,m.migmodel,m.sizemodel,m.npops,pars,m.splitprop);
    fprintf(out,"%g %g %g %g "
	    ,pars[4],pars[5],t.simroot->time,calc_simlength(t.simroot));
    if (m.sizemodel) 
      fprintf(out,"%g %g %g "
	      ,pars[6],pars[1],log(pars[5]) + pars[5]*pars[6]*pars[1]);
    if (m.migmodel) {
      for (j=1;j<=t.npops;j++) fprintf(out,"%g ",t.populations[j].relpopsize);
      for (j=1;j<t.npops-1;j++) 
	fprintf(out,"%d %g ",t.populations[t.npops+j].location,
		t.populations[t.npops+j].time);
      fprintf(out,"%g ",	t.populations[t.npops+j].time);	
    } 
    fprintf(out,"\n");
    destroy_poptree(&t);
  }
  fprintf(stderr,"\ncompleted sample\n");
}
/*************************************************************************/
