/* version1.0 (c) Copyright 2000 by Ian Wilson
	 Written by Ian Wilson.
	 Permission is granted to copy and use this program provided no fee is
	 charged for it and provided that this copyright notice is not removed. */
#include <stdio.h>
#include "nrutil.h"
#include "cutil.h"
#include "mutate.h"
#include "random.h"
#include <math.h>
/****************************************************************************/
void printSTRgeno(haplotype hap, int nSTR, int first)
{
		int locus;
		for (locus=first+1;locus<=first+nSTR;locus++) fprintf(stdout, "%d ", hap[locus]);
		fprintf(stdout, "\n");
}
/***************************************************************************/
/*	STR mutation down the left and right branches of a node								*/
/***************************************************************************/
void mutateleftrightSTR(simnode *from, int nSTR,int first, double theta)
{
		int locus,nmut;
	
		for (locus=first+1;locus<=nSTR+first;locus++) {
		nmut = (int)poidev(theta*(from->time-from->descendent1->time)/2.);
		if (nmut>0)
			from->descendent1->haplotype[locus]
			=from->haplotype[locus]+2*(int)bnldev(0.5,nmut)-nmut;
		else from->descendent1->haplotype[locus]=from->haplotype[locus];
		
		nmut = (int)poidev(theta*(from->time-from->descendent2->time)/2.);
		if (nmut>0)
			from->descendent2->haplotype[locus]
			=from->haplotype[locus]+2*(int)bnldev(0.5,nmut)-nmut;
		else from->descendent2->haplotype[locus]=from->haplotype[locus];
		}
}
/****************************************************************************/
void getinitialSTRgeno(haplotype hap, int nSTR,int first)
{
		int locus;
		for (locus=first+1;locus<=first+nSTR;locus++) hap[locus]=1;
}
/***************************************************************************/
void mutateSTR(simnode *sample, int ss,int nSTR,int ninf, double theta)
/* this mutation function depends on the fact that the times are in order */
{
		int i,j,*mini;
		int locus;

		getinitialSTRgeno(sample[2*ss-1].haplotype,nSTR,ninf);
		
		for (i=2*ss-1;i>ss;i--) {
			mutateleftrightSTR((&sample[i]),nSTR,ninf,theta);
		}
		
		/*
		fprintf(stdout, "STRgenos:\n");
		for (i=2*ss-1;i>ss;i--) {
			mutateleftrightSTR((&sample[i]),nSTR,ninf,theta);
			fprintf(stdout, "\t");
			printSTRgeno(sample[i].haplotype, nSTR, ninf);
		}
		*/
	
		mini = nrivector(1,nSTR+ninf);
		
		for (i=ninf+1;i<=nSTR+ninf;i++) {
			mini[i]=sample[ss].haplotype[i];
			
			for (j=1;j<2*ss;j++) {
				if (sample[j].haplotype[i]<mini[i]) {
					mini[i] = sample[j].haplotype[i];
				}
			}
			
			if (mini[i]<1) {
				for (j=1;j<2*ss;j++) {
					sample[j].haplotype[i]-=mini[i]-1;
				}
			}
		}
		free_ivector(mini,1);
		
		return;
}
/***************************************************************************/
/*	Change the infinite sites mutation type below a node									 */
/***************************************************************************/
void changebelow(simnode *any, int which)

{
		any->haplotype[which]=1;
		if (any->descendent1!=NULL)
		changebelow(any->descendent1,which);
		if (any->descendent2!=NULL)
		changebelow(any->descendent2,which);
}
/***************************************************************************/
simnode *findnodemut(simnode *any, double *muttime)
{
		simnode *retnode=NULL;
	
		if (any->descendent1!=NULL) {
		if (any->time-any->descendent1->time>=*muttime)
			retnode= any->descendent1;
		else {
			*muttime -= any->time-any->descendent1->time;
			retnode = findnodemut(any->descendent1,muttime);
		}
		}
		if (retnode!=NULL) return retnode;
		if (any->descendent2!=NULL) {
		if (any->time-any->descendent2->time>=*muttime)
			retnode= any->descendent2;
		else {
			*muttime -= any->time-any->descendent2->time;
			retnode = findnodemut(any->descendent2,muttime);
		}
		}
		return retnode;
}
/***************************************************************************/
/*	performs both infinite sites and STR mutations												 */
/***************************************************************************/
void mutate(simnode *sample, simmodel *m)
{
		int i;
		simnode *mutnode;
		double length=0.0,muttime;

		for (i=2*m->n-1;i>m->n;i--)
		length += (double)(2*m->n+1-i)*(sample[i].time-sample[i-1].time);
		if (m->inftheta>0.0) {
		for	(;;) {
			m->infsites = (int)poidev(length*m->inftheta/2.0);
			if ((m->infsites)||(m->nSTR)) break;
			printf(" warning no infinite sites mutations, repeating\n");
		}
		}
		setuphaplotypes(sample,m->n,m->infsites,m->nSTR);
	
		if (m->nSTR) mutateSTR(sample,m->n,m->nSTR,m->infsites,m->theta);
		if (m->infsites) {	
		for (i=1;i<=m->infsites;i++) {
			muttime=ranDum()*length;
			mutnode = findnodemut(&(sample[2*m->n-1]),&muttime);
			if (mutnode==NULL) nrerror("WTH"); /* should never happen */
			changebelow(mutnode,i);
		}	 	
		}
}
/***************************************************************************/

