#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <ctype.h>
#include "prior.h"
#include "nrutil.h"
#include "random.h"
#include "sample.h"
#include "cutil.h"
#include "myio.h"
/*************************************************************************/
void getvalues(double prs[7], simmodel *m)
{
	double alpha,N,beta,kappa,split,mu;

	if (m->alphaprior.prtype!=NULLPRIOR)
		sample_prior(&alpha,m->alphaprior);
	if (m->Nprior.prtype!=NULLPRIOR)
		sample_prior(&N,m->Nprior);
	if (m->muprior.prtype!=NULLPRIOR)
		sample_prior(&mu,m->muprior);
	if (m->kappaprior.prtype!=NULLPRIOR) {
		sample_prior(&kappa,m->kappaprior);
	} else kappa=-1;
	if (m->betaprior.prtype!=NULLPRIOR) {
		sample_prior(&beta,m->betaprior);
	} else beta=-1;

	if (m->splitprior.prtype!=NULLPRIOR) {
		sample_prior(&split,m->splitprior);
	} else split=m->splittime;

	

	if (beta<0.0) {
		beta = kappa/(N*alpha);
	} 
	if (m->Nprior.prtype!=NULLPRIOR) { 
	prs[2]=2.*N*mu;
	} else prs[2]=m->theta;
	prs[0]=N*alpha;
	prs[1]=beta;
	prs[3]=split;
	prs[4]=mu;
	prs[6]=alpha;
	prs[5]=N;
}


/*************************************************************************/
simmodel readsimmodel( char *file)
{
  simmodel m;
  FILE *fd;
  int i;
  prior nllprior={NULLPRIOR,{0.0,0.0}};
    
  fd=openinputfile(file);
    
  /**** The random number seed                            */
  m.seed=int_scan_b(fd,"seed",1,quiet);
  /*  infinite sites parameters          *******************/
  m.inftheta=double_scan_b(fd,"inftheta",-1.0,quiet);
  m.infsites=int_scan_b(fd,"infsites",0,quiet);

  if ((m.inftheta>0.0)&&(m.infsites>0)) {
    printf("both inftheta and infsites set, using infsites");
    m.inftheta=-1.0;
  }
  /*********************************************************/
  m.nSTR=int_scan_b(fd,"nSTR",0,quiet);

  m.height=double_scan_b(fd,"height",-1.0,quiet);
  m.migmodel=int_scan_b(fd,"migmodel",0,quiet);

  if (m.migmodel==1) {
    if (findstart(fd,"samplesize")) {
      m.ss=readintegerline(fd);
      m.npops=m.ss[0];
      m.n=0;
      for (i=1;i<=m.npops;i++)
	m.n+=m.ss[i];
      m.ss[0]=m.n;
    }
    else nrerror("must have samplesizes for multiple pop model\n");
    m.splittime=double_scan_b(fd,"splittime",0.5,quiet);
    m.sizepars[3]=m.splittime;
    if (findstart(fd,"splitprop")) {
      m.splitprop=readdoubleline(fd,&i); 
      if (i!=m.npops)
	nrerror("splitprop has too few entries");
    }
    else {
      printf("splitprop not in %s, using random proportions\n",file);
      m.splitprop=NULL;//nrdvector(1,m.npops);
      //for (i=1;i<=m.npops;i++) m.splitprop[i]=1/(double)m.npops;
    }
  } else if (m.migmodel==2) {  /* this migmodel for priors  */
    if (findstart(fd,"samplesize")) {
      m.ss=readintegerline(fd);
      m.npops=m.ss[0];
      m.n=0;
      for (i=1;i<=m.npops;i++)
	m.n+=m.ss[i];
    }
    else nrerror("must have sample sizes for multiple pop model\n");
  } else {
    if (findstart(fd,"samplesize"))
      m.n=nextint(fd);
    else 
      printf("no samplesize given in %s, using %d\n",file,m.n=50);
    m.ss=nrivector(0,1);
    m.ss[0]=m.n;
  }
  m.sizemodel=int_scan_b(fd,"sizemodel",0,quiet);
  m.theta=double_scan_b(fd,"theta",10,quiet);
  if (m.sizemodel) {
    m.R=double_scan_b(fd,"growthrate",10,quiet);
    if (m.sizemodel==2) 
      m.Rtime=double_scan_b(fd,"growthtime",0.5,quiet);
    m.sizepars[0]=m.R;
    m.sizepars[1]=m.Rtime;
    m.sizepars[2]=exp(m.R*m.Rtime);
  } else m.R=0.0;

  m.muprior=prior_scan(fd,"muprior","null",loud);
  m.Nprior=prior_scan(fd,"Nprior","null",loud);
  m.alphaprior=prior_scan(fd,"alphaprior","null",loud);
  m.betaprior=prior_scan(fd,"betaprior","null",loud);
  m.kappaprior=prior_scan(fd,"kappaprior","null",loud);
  m.splitprior=prior_scan(fd,"splitprior","null",loud);
  /**** The number of replicates (only for prior)          *
   *  need to be careful (because of samplesize)           */
  if (findstart(fd,"samples:"))  fscanf(fd,"%d",&m.replicates);

  return m;
}
/***************************************************************************/
void setuphaplotypes(simnode *sample,int ss, int ninf, int nSTR)
{
  int i;
	
  if (ninf+nSTR>0) {
    for (i=1;i<2*ss;i++) 
      sample[i].haplotype=nrivector0(1,nSTR+ninf);
  } else for (i=1;i<2*ss;i++) sample[i].haplotype=NULL;	
  return;    
} 
/***************************************************************************/
/*
 * A set of functions for simulating trees and subsampling from already drawn
 * trees
 */
/***********************************************************************/
double calc_simlength(simnode *anynode)
{
  double temp=0.0;
	
  if (anynode->descendent1==NULL) temp += anynode->time;
  else temp += calc_simlength(anynode->descendent1) + 
	 anynode->time-anynode->descendent1->time;
	
  if (anynode->descendent2==NULL) temp += anynode->time;
  else temp += calc_simlength(anynode->descendent2) + 
	 anynode->time-anynode->descendent2->time;
	
  return temp;
}
/**********************************************************************/
void writepopnode(FILE *out,popnode *any)
{
  fprintf(out, "%d", any->location );	
}
/***********************************************************************/
void writepoputil(FILE *out,popnode *any)
{
    fprintf(out, "(");
    if (any->left->left != NULL) 
		writepoputil(out,any->left);
   writepopnode(out,any->left);
    fprintf(out, ":%10.6lg",any->time - any->left->time);
    fprintf(out,",");

    if (any->right->left != NULL)	
	writepoputil(out,any->right);
    writepopnode(out,any->right);
    fprintf(out, ":%10.6lg",any->time - any->right->time);
    fprintf(out, ")");
}
/***********************************************************************/
void write_Newickpoptree(popnode *root, FILE *out)
{
    writepoputil(out,root);
    writepopnode(out,root);
    fprintf(out,";\n");
    return;
}
/***********************************************************************/
void writenode(FILE *out,simnode *any, int npop, int ninf, int nstr, int *label)
{

    int i;

    if (*label!=0) {
	  fprintf(out,"'{%d} ",*label);
	  if (any->descendent1==NULL) *label +=1;
    } else    fprintf(out,"'");
    if (npop>1)  fprintf(out, "<%d> ", any->location);
    for (i=1;i<=ninf;i++) {
      fprintf(out,"%d",any->haplotype[i]);
    }
    if (ninf>0) 	fprintf(out,"~");
    for (i = ninf+1; i < ninf+nstr; i++)
      fprintf(out,"%d-",any->haplotype[i]);
    if (nstr>0) fprintf(out,"%d",any->haplotype[ninf+nstr]);
    fprintf(out,"'");	
}
/***********************************************************************/
void writeutil(simnode *anynode, FILE *out, int npop,int ninf, int nstr, int *label)
{
    fprintf(out, "(");
    if (anynode->descendent1->descendent1 != NULL) 
	writeutil(anynode->descendent1, out,npop,ninf,nstr,label);
    writenode(out,anynode->descendent1,npop,ninf,nstr,label);
    fprintf(out, ":%10.6lg",anynode->time - anynode->descendent1->time);
    fprintf(out,",");
    if (anynode->descendent2->descendent1 != NULL)	
	writeutil(anynode->descendent2, out,npop,ninf,nstr,label);
    writenode(out,anynode->descendent2,npop,ninf,nstr,label);
    fprintf(out, ":%10.6lg",anynode->time - anynode->descendent2->time);
    fprintf(out, ")");
}
/***********************************************************************/
void write_Newicksimtree(FILE *output, simnode *root,
						 int migmodel,int nloci,int ninf)
{
  int label=0;
    writeutil(root, output,migmodel+1,ninf,nloci,&label);
    writenode(output,root,migmodel+1,ninf,nloci,&label);
    fprintf(output,";");
    return;
}
/***********************************************************************/
