#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "prior.h"
#include "random.h"
#include "nrutil.h"
#include "sample.h"
#include "cutil.h"
#include "myio.h"
#include "simtree.h"
#include "mutate.h"
void getvalues(double prs[4], simmodel *m);
/***********************************************************************/
int main(int argc, char *argv[])
{
    simmodel m;
    poptree t;
    FILE *out;
    int i,j,**haplotypes,count=1;
    char outfile[30],infile[30];
    double pars[7];
  
    if (argc==1) {
      printf("no arguments entered:\n using sample <infile> <out> <seed>\n");
      strcpy(infile,"infile");
      strcpy(outfile,"outfile");
    } else {
      printf("input line is ");
      for (i=0;i<argc;i++) printf("%s ",argv[i]);
      printf("\n");
      if ((argc<3)||(argc>4)) {
	fprintf(stderr,"use:\n%s infile outfile (seed)\n"
		,argv[0]);  
	exit(0);
      }
      strcpy(infile,argv[1]);
      strcpy(outfile,argv[2]);
    }
    
    m = readsimmodel(infile);
    if (argc==4) m.seed = atoi(argv[3]);
    starTup(m.seed);
    for (;;) {
      getvalues(pars,&m);
      m.theta=pars[2];
      t = simulate_poptree(m.ss,m.migmodel,m.sizemodel,m.npops,pars,m.splitprop);
      if (m.height>0.0) {
	if (fabs((t.simroot->time-m.height)/m.height)<0.001) break;
	printf("got tree height %g\n",t.simroot->time);
	destroy_poptree(&t);
      } else break;
    }
    if (m.nSTR+m.infsites>0) mutate(t.populations[0].sample,&m);

    out=openstemleafoutputfile(outfile,"data");
    write_Newicksimtree(out,t.simroot,m.migmodel,m.nSTR,m.infsites);
    fprintf(out,"\n\nsummary\n");
    fprintf(out,"%d samples with %d STR loci and %d infinite sites\n",
		m.n,m.nSTR,m.infsites);
	if (m.Nprior.prtype!=NULLPRIOR)
    fprintf(out,"mu %g\nN %g\n",pars[4],pars[5]);
	if (m.sizemodel>0) fprintf(out,"alpha %g\n",pars[6]);
	if (m.sizemodel>1) fprintf(out,"beta %g\n",pars[1]);
    fprintf(out,"theta %g\n",m.theta);
    if (m.sizemodel==1) fprintf(out,"exp model with parameter %g\n",m.R);
    else if (m.sizemodel==2) fprintf(out,"exp two par model %g %g\n",
		pars[0],pars[1]);
    fprintf(out,"height %g length %g\n",t.simroot->time,calc_simlength(t.simroot));

	if (m.infsites) {
		fprintf(out,"root ");
		for (i=1;i<=m.infsites;i++) 
			fprintf(out,"%d ",t.simroot->haplotype[i]);
		fprintf(out,"\n");
	}
	if (m.migmodel) {
		fprintf(out,"Rshape ");
		for (j=1;j<t.npops-1;j++) 
			fprintf(out,"%d %g ",t.populations[t.npops+j].location,
			t.populations[t.npops+j].time);
		fprintf(out,"%g ",	t.populations[t.npops+j].time);
		fprintf(out,"\n");
		
		write_Newickpoptree(t.root,out);
		fprintf(out,"proportions: ");
		for (i=1;i<=m.npops;i++) 
			fprintf(out,"%g ",t.populations[i].relpopsize);
		fprintf(out,"\n");
	}
	if (m.migmodel) write_Newickpoptree(t.root,stdout);

	//	output_line();
	
	fclose(out);
	
    haplotypes = (int**)MALLOC((m.n+1)*sizeof(int *));
    if (!haplotypes) nrerror("error allocating STRgeno");
	if (m.migmodel) {
		for (i=1;i<=m.npops;i++) {
			for (j=1;j<=t.populations[i].lines;j++) {
				haplotypes[count++]=t.populations[i].sample[j].haplotype;
			}
		}
	} else {
		for (i=1;i<=m.n;i++) {
			haplotypes[i]=t.populations[0].sample[i].haplotype;
		}
	}
	out=openoutputfile(outfile);
    write_imatrix(out,haplotypes,1,m.n,1,m.nSTR+m.infsites);
    fclose(out);
   // write_imatrix(stdout,haplotypes,1,m.n,1,m.nSTR+m.infsites);
    if (m.migmodel) {
      openstemleafoutputfile(outfile,"loc");
      for (i=1;i<=t.npops;i++) 
	for (j=1;j<=t.populations[i].lines;j++)
	  fprintf(out,"%d\n",i);
      fclose(out);
    }
	
    FREE(haplotypes); 
    destroy_poptree(&t);

    // fprintf(stderr,"\ncompleted sample\n");
    return EXIT_SUCCESS;
}

