#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "nrutil.h"
#include "random.h"
#include "cutil.h"
#include "simtree.h"


/***************************************************************************/
/***************************************************************************/
/*   Generate the next time                                                */
/***************************************************************************/
/***************************************************************************/
double getnexttimeprop(double last,double left,int gtype,double *a, double pr)
{
  double t;
  if (gtype==0) {
    return -LOG(ranDum())*2.0*pr/(left*(left-1.0));
  } else if (gtype==1) {
    if (fabs(a[0])>1E-3) {
      return LOG(
		 exp(a[0]*last)-LOG(ranDum())*2.*a[0]*pr/
		 (left*(left-1.0)) )/a[0] -last;
    }
    else {
      return -LOG(ranDum())*2.0*pr/(left*(left-1.0));
    }
  } else if (gtype==2) {
    if (last>=a[1])
      return -LOG(ranDum())*2.0*pr/(left*(left-1.0));
    else {
      t = LOG(
	      exp(a[0]*last)-LOG(ranDum())*exp(a[0]*a[1])*2.*a[0]*pr
	      /(left*(left-1.0)))/a[0]-last;
      if (t+last> a[1]) 
	return a[1] + getnexttimeprop(a[1],left,gtype,a,pr);
      else return t;
    }	
  } else {
    printf("not written yet in getnexttimechange");
    exit(-1);
    return 0.0;
  }
}
/***************************************************************************/
simnode *getshape1pop(int n, int sizem, double *pars)
{
  simnode *sample,**whichnode;
  int i,pick1,pick2;
  double n_left,add,t=0.0;

  sample =  (simnode *) MALLOC((2 * n) * sizeof(simnode));
  if (!sample) nrerror("error allocating sample");
  whichnode = (simnode **) MALLOC((n + 1) * sizeof(simnode *));
  if (!whichnode) nrerror("error allocating whichnode");
  for (i = 1; i <= n; i++) {
    sample[i].descendent1 = NULL;
    sample[i].descendent2 = NULL;
    sample[i].time = 0.0;
    whichnode[i] = &(sample[i]);
  }
  for (i = 1; i < n ; i++) {
    n_left = (double) (n + 1 - i);
    add = getnexttimeprop(t,n_left,sizem,pars,1.0);
    t += add;
    pick1 = 1 + (int) ((n_left) * ranDum());
    pick2 = 1 + (int) ((n_left-1.) * ranDum());
    if (pick2>=pick1) pick2+=1;
    sample[n+i].time = t;
    sample[n+i].descendent1 = whichnode[pick1];
    sample[n+i].descendent2 = whichnode[pick2];
		
    whichnode[pick2] = &(sample[n+i]);
    whichnode[pick1] = whichnode[n + 1 - i];
  }
  FREE(whichnode);
  printf("t     = %g\n", t);
  return sample;
}
/***************************************************************************/
simnode *getshapefiss(int n,int *ss, int npops, double splittime, 
		      double *splitprop,int sizem,double *pars)
{
  simnode **whichnode,*sample;
  int totleft=0,location,i,pick1,pick2,count,base,left;
  double t,add,n_left;
	
  sample = (simnode *) MALLOC((2 * n) * sizeof(simnode));
  if (!sample) nrerror("error allocating data");
  whichnode = (simnode **) MALLOC((n + 1) * sizeof(simnode *));
  if (!whichnode) nrerror("error allocating whichnode");
	
  count=1;
  for (location=1;location<=npops;location++) {
    for (i=1;i<=ss[location];i++) {
      sample[count].descendent1 = NULL;
      sample[count].descendent2 = NULL;
      sample[count].time = 0.0;
      whichnode[count] = &(sample[count]);
      sample[count++].location = location;
    }
  }
  count=1;base=0;
  for (location=1;location<=npops;location++) {
    /*  loop through each of the locations */
    t=0.0;
    n_left = (double)ss[location];
    for (i=1;i<ss[location];i++) {
      add = getnexttimeprop(t,n_left,sizem,pars,splitprop[location]);
      if (t+add>splittime) break;
      t += add;
      pick1 = 1 + (int) ((n_left) * ranDum());
      pick2 = 1 + (int) ((n_left-1.0) * ranDum());
      if (pick2>=pick1) pick2+=1;
			
      sample[n+count].time = t;
      sample[n+count].descendent1 = whichnode[base+pick1];
      sample[n+count].descendent2 = whichnode[base+pick2];
      sample[n+count].location = location;
			
      whichnode[base+pick2] = &(sample[n+count]);
      whichnode[base+pick1] = whichnode[base+ss[location]+1-i];
      n_left -=1.0;
      count += 1;
    }
    left = (int)(n_left+0.01);
    if (location>1) {
      for (i=1;i<=left;i++) 
	whichnode[totleft+i] = whichnode[base+i];
    }
    base += ss[location];
    totleft+=left;
  }
  t=splittime;
  for (i=1;i<totleft;i++) {
    n_left = (double) (totleft + 1 - i);
    add = getnexttimeprop(t,n_left,sizem,pars,1.09);
    t += add;
    pick1 = 1 + (int) ((n_left) * ranDum());
    pick2 = 1 + (int) ((n_left-1.) * ranDum());
    if (pick2>=pick1) pick2+=1;
		
    sample[n+count].time = t;
    sample[n+count].descendent1 = whichnode[pick1];
    sample[n+count].descendent2 = whichnode[pick2];
		
    whichnode[pick2] = &(sample[n+count++]);
    whichnode[pick1] = whichnode[totleft +1 - i];
  }
  FREE(whichnode);
  return sample;
}
/***************************************************************************/
int recurse_simulate_pt(popnode *pn,simnode *ancestors,simnode **which,
			double *pars,int sizemodel, int *ancpos)
{
  int linesleft,linesright,count,d1,d2,i;
  double nleft,time,add,toptime;
  simnode *anc;
    
  /*   printf("location %d ancpos = %d ",pn->location,*ancpos);
       if (pn->left) printf("left location %d right location %d\n",pn->left->location,pn->right->location);
       else printf("\n");*/
    
  if (pn->left==NULL) {
    time=0.0;
    for (i=1;i<=pn->lines;i++) which[i]=&(pn->sample[i]);
  } else {
    time=pn->time;
    linesleft=
      recurse_simulate_pt(pn->left,ancestors,which,pars,sizemodel,ancpos);
    linesright=recurse_simulate_pt(pn->right,ancestors,
				   which+linesleft,pars,sizemodel,ancpos);
    pn->lines=linesleft+linesright;
  }
  if (pn->up==NULL) toptime=1E22;
  else toptime=pn->up->time;
  anc = ancestors + *ancpos;
	
  nleft=(double)pn->lines;
  for (count=1;count<pn->lines;count++) {
    add=getnexttimeprop(time,nleft,sizemodel,pars,pn->relpopsize);
    /* printf("%d base %g top %g ta %g n %g last %g add = %g\n",
       pn->location,pn->time,toptime,pars[1],nleft,time,add);*/
    if (time+add>toptime) break;
    time += add;
    d1=1+(int)(ranDum()*nleft);
    d2=1+(int)(ranDum()*(nleft-1.));
    if (d2>=d1) d2++;
    anc[count].time=time;
    anc[count].descendent1=which[d1];
    anc[count].descendent2=which[d2];
    anc[count].location=
      which[d1]->location|which[d2]->location;
    which[d1]=&(anc[count]);
    which[d2]=which[(int)(nleft+0.5)];
    nleft-=1.;
  }
  if (count==pn->lines) {
    /*	printf(" %d coals %d left\n",pn->lines-1,1);*/
    *ancpos += pn->lines-1;
    return 1;
  }
  else {
    /* printf(" %d coalescences %d left\n",count-1,(int)(nleft+0.5));*/
    *ancpos += count-1;
    return (int)(nleft+0.5);
  }

}
/*************************************************************/
poptree conv_index_poptree(int *index, double *time, double *proportions,
			   int *ss,int npops)
{
  int i,j,pm;
  popnode **whpn;
  poptree npt;
    
  npt.populations=(popnode *)MALLOC((2*npops)*sizeof(popnode));
  if (!npt.populations) {
    printf("error allocating in conv...");
    exit(-1);
  }
  whpn =(popnode **)MALLOC((npops+1)*sizeof(popnode *));
  if (!whpn) {
    printf("error allocating in conv...");
    exit(-1);
  }
  npt.populations[0].lines=ss[0];
  for (i=1;i<=npops;i++)  {
    npt.populations[i].location=1<<(index[i]-1);
    npt.populations[i].relpopsize=proportions[i];
    npt.populations[i].lines=ss[i];
    npt.populations[i].left=NULL;
    npt.populations[i].right=NULL;
    npt.populations[i].time=0.0;
    whpn[i]=&(npt.populations[i]);
  }
  npt.npops=npops;
    
  for (i=1;i<npops;i++) {
    pm = posmin(time,npops-i);
    npt.populations[npops+i].left=whpn[pm];
    npt.populations[npops+i].right=whpn[pm+1];
    whpn[pm]->up=&(npt.populations[npops+i]);
    whpn[pm+1]->up=&(npt.populations[npops+i]);
    npt.populations[npops+i].time=time[pm];
		
    npt.populations[npops+i].relpopsize=
      whpn[pm]->relpopsize+whpn[pm+1]->relpopsize;
    npt.populations[npops+i].location=
      whpn[pm]->location|whpn[pm+1]->location;
    whpn[pm]=&(npt.populations[npops+i]);
    for (j=pm;j<npops-i;j++)
      time[j]=time[j+1];
    for (j=pm+1;j<npops+1-i;j++)
      whpn[j]=whpn[j+1];
  }
  npt.root=&(npt.populations[2*npt.npops-1]);
  npt.root->up=NULL;
  FREE(whpn);
  return npt;
}
/*********************************************************************/
void startingtree(poptree *pt)
{
  int i,j,count,location;
	
  pt->populations[0].sample=
    (simnode *)MALLOC((2*pt->populations[0].lines)*sizeof(simnode));
  if (!pt->populations[0].sample) {
    printf("error allocating starting tree");
    exit(-1);
  }
  count=0;
  for (i=1;i<=pt->npops;i++) {
    location = 1<<(i-1);
    pt->populations[i].sample=pt->populations[0].sample+count;
    for (j=1;j<=pt->populations[i].lines;j++) {
      count +=1;
      pt->populations[i].sample[j].descendent1=
	pt->populations[i].sample[j].descendent2=NULL;
      pt->populations[i].sample[j].location=location;
      pt->populations[i].sample[j].time=0.0;
    }
  }
}
/*******************************************************************/
poptree simulatepoptree2(int *n, int npops, double *pars, int sizemodel, double *prop)
{
  double *times,*proportions;
  int *index,i,which;
  poptree temp;
  simnode **whichnode;
	
  times=(double *)MALLOC((npops)*sizeof(double));
  proportions=(double *)MALLOC((npops+1)*sizeof(double));
  index=(int*)MALLOC((npops+1)*sizeof(int)); 
  for (i=1;i<=npops-1;i++) times[i]=ranDum()*pars[3];
  which=1+(int)(ranDum()*(double)(npops-1.0));
  times[which]=pars[3];
  for (i=1;i<=npops;i++) 	index[i]=i;
    
  if (prop==NULL) {
    rdirichlet(proportions,2.0,npops);
  } else {
    for (i=1;i<=npops;i++) proportions[i]=prop[i];
  }
  permute_integers(index,npops);
  temp=conv_index_poptree(index,times,proportions,n,npops);
  FREE(index);
  FREE(proportions);
  FREE(times);
	
  startingtree(&temp);
  whichnode=(simnode **)MALLOC((n[0]+1)*sizeof(simnode *));
  i=0;
  which=recurse_simulate_pt(temp.root,temp.populations[0].sample+n[0],
			    whichnode,pars,sizemodel,&i);
  FREE(whichnode);
  if (which != 1) {
    printf("%d left incorrect coalescences",which);
    exit(0);
  }
  return temp;
}
/***************************************************************************/
poptree simulate_poptree(int *n,int migmodel, int model, int npops, double *pars, double *proportions)
{
  poptree tmp;
	
  if (migmodel==0) {
    tmp.populations=(popnode*)MALLOC(sizeof(popnode));
    if (!tmp.populations) {
      printf("error allocating poptree");
      exit(-1);
    }
    tmp.populations[0].sample = getshape1pop(n[0],model,pars);
  } else if (migmodel==1)  {
    tmp=simulatepoptree2(n,npops,pars,model,proportions);
  }
  else printf("this mig model not defined yet");
  tmp.simroot=&(tmp.populations[0].sample[2*n[0]-1]);
  return tmp;    
}
/***********************************************************************/
void destroy_poptree(poptree *any)
{
  FREE(any->populations[0].sample);
  FREE(any->populations);
}
/***********************************************************************/
